@@ -31,6 +31,7 @@ are not tied to any specific part of gPodder.
 """
 import collections
 import datetime
+import email
 import glob
 import gzip
 import http.client
@@ -55,23 +56,20 @@ import threading
 import time
 import urllib.error
 import urllib.parse
-import urllib.request
 import webbrowser
 import xml.dom.minidom
-from html.entities import entitydefs
+from html.entities import entitydefs, name2codepoint
+from html.parser import HTMLParser
+
+import requests
+import requests.exceptions
+from requests.packages.urllib3.util.retry import Retry
 
 import gpodder
 
 logger = logging.getLogger(__name__)
 
 
-if sys.hexversion < 0x03000000:
-    from html.parser import HTMLParser
-    from html.entities import name2codepoint
-else:
-    from html.parser import HTMLParser
-    from html.entities import name2codepoint
-
 try:
     import html5lib
 except ImportError:
@@ -605,7 +603,7 @@ def is_html(text):
     >>> is_html('a < b < c')
     False
     """
-    html_test = re.compile('<[a-z][a-z0-9]*(?:\s.*?>|\/?>)', re.IGNORECASE | re.DOTALL)
+    html_test = re.compile(r'<[a-z][a-z0-9]*(?:\s.*?>|\/?>)', re.IGNORECASE | re.DOTALL)
     return bool(html_test.search(text))
 
 
@@ -619,18 +617,18 @@ def remove_html_tags(html):
         return None
 
     # If we would want more speed, we could make these global
-    re_strip_tags = re.compile('<[^>]*>')
-    re_unicode_entities = re.compile('&#(\d{2,4});')
-    re_html_entities = re.compile('&(.{2,8});')
-    re_newline_tags = re.compile('(<br[^>]*>|<[/]?ul[^>]*>|</li>)', re.I)
-    re_listing_tags = re.compile('<li[^>]*>', re.I)
+    re_strip_tags = re.compile(r'<[^>]*>')
+    re_unicode_entities = re.compile(r'&#(\d{2,4});')
+    re_html_entities = re.compile(r'&(.{2,8});')
+    re_newline_tags = re.compile(r'(<br[^>]*>|<[/]?ul[^>]*>|</li>)', re.I)
+    re_listing_tags = re.compile(r'<li[^>]*>', re.I)
 
     result = html
 
     # Convert common HTML elements to their text equivalent
     result = re_newline_tags.sub('\n', result)
     result = re_listing_tags.sub('\n * ', result)
-    result = re.sub('<[Pp]>', '\n\n', result)
+    result = re.sub(r'<[Pp]>', '\n\n', result)
 
     # Remove all HTML/XML tags from the string
     result = re_strip_tags.sub('', result)
@@ -642,7 +640,7 @@ def remove_html_tags(html):
     result = re_html_entities.sub(lambda x: entitydefs.get(x.group(1), ''), result)
 
     # Convert more than two newlines to two newlines
-    result = re.sub('([\r\n]{2})([\r\n])+', '\\1', result)
+    result = re.sub(r'([\r\n]{2})([\r\n])+', '\\1', result)
 
     return result.strip()
 
@@ -661,7 +659,7 @@ class HyperlinkExtracter(object):
         for target, parts in group_it:
             t = ''.join(text for _, text in parts if text is not None)
             # Remove trailing spaces
-            t = re.sub(' +\n', '\n', t)
+            t = re.sub(r' +\n', '\n', t)
             # Convert more than two newlines to two newlines
             t = t.replace('\r', '')
             t = re.sub(r'\n\n\n+', '\n\n', t)
@@ -1187,31 +1185,27 @@ def url_add_authentication(url, username, password):
     return urllib.parse.urlunsplit(url_parts)
 
 
-def urlopen(url, headers=None, data=None, timeout=None):
+def urlopen(url, headers=None, data=None, timeout=None, **kwargs):
     """
     An URL opener with the User-agent set to gPodder (with version)
     """
-    username, password = username_password_from_url(url)
-    if username is not None or password is not None:
-        url = url_strip_authentication(url)
-        password_mgr = urllib.request.HTTPPasswordMgrWithDefaultRealm()
-        password_mgr.add_password(None, url, username, password)
-        handler = urllib.request.HTTPBasicAuthHandler(password_mgr)
-        opener = urllib.request.build_opener(handler)
-    else:
-        opener = urllib.request.build_opener()
-
     if headers is None:
         headers = {}
     else:
         headers = dict(headers)
 
+    if not timeout:
+        timeout = gpodder.SOCKET_TIMEOUT
+
+    retry_strategy = Retry(
+        total=3,
+        status_forcelist=Retry.RETRY_AFTER_STATUS_CODES.union((408, 418, 504, 598, 599,)))
+    s = requests.Session()
+    a = requests.adapters.HTTPAdapter(max_retries=retry_strategy)
+    s.mount('http://', a)
+    s.mount('https://', a)
     headers.update({'User-agent': gpodder.user_agent})
-    request = urllib.request.Request(url, data=data, headers=headers)
-    if timeout is None:
-        return opener.open(request)
-    else:
-        return opener.open(request, timeout=timeout)
+    return s.get(url, headers=headers, data=data, timeout=timeout, **kwargs)
 
 
 def get_real_url(url):
@@ -1219,7 +1213,7 @@ def get_real_url(url):
     Gets the real URL of a file and resolves all redirects.
     """
     try:
-        return urlopen(url).geturl()
+        return urlopen(url).url
     except:
         logger.error('Getting real url for %s', url, exc_info=True)
         return url
@@ -1805,8 +1799,7 @@ def get_update_info():
         (False, '3.0.5', '2012-02-29', 10)
     """
     url = 'https://api.github.com/repos/gpodder/gpodder/releases/latest'
-    data = urlopen(url).read().decode('utf-8')
-    info = json.loads(data)
+    info = urlopen(url).json()
 
     latest_version = info.get('tag_name', '').replace('gpodder-', '')
     release_date = info['published_at']
@@ -1831,17 +1824,22 @@ def run_in_background(function, daemon=False):
 
 
 def linux_get_active_interfaces():
-    """Get active network interfaces using 'ip link'
+    """Get active network interfaces using 'ip addr'
 
-    Returns a list of active network interfaces or an
-    empty list if the device is offline. The loopback
-    interface is not included.
+    A generator function yielding network interface
+    names with an inet (or inet6) and a broadcast
+    address, indicating an active network connection
     """
-    process = Popen(['ip', 'link'], close_fds=True, stdout=subprocess.PIPE)
-    data, _ = process.communicate()
-    for interface, _ in re.findall(r'\d+: ([^:]+):.*state (UP|DORMANT|UNKNOWN)', data.decode(locale.getpreferredencoding())):
-        if interface != 'lo':
-            yield interface
+    process = Popen(
+        ['ip', 'addr', 'show', 'scope', 'global', 'up'],
+        close_fds=True, stdout=subprocess.PIPE)
+    data, x = process.communicate()
+    for record in re.split(r'^\d+: ',
+                           data.decode(locale.getpreferredencoding()),
+                           flags=re.MULTILINE):
+        mo = re.match(r'^([^:]*):.*inet.*brd', record, flags=re.DOTALL)
+        if mo:
+            yield mo.group(1)
 
 
 def osx_get_active_interfaces():
@@ -1853,8 +1851,8 @@ def osx_get_active_interfaces():
     """
     process = Popen(['ifconfig'], close_fds=True, stdout=subprocess.PIPE)
     stdout, _ = process.communicate()
-    for i in re.split('\n(?!\t)', stdout.decode('utf-8'), re.MULTILINE):
-        b = re.match('(\\w+):.*status: (active|associated)$', i, re.MULTILINE | re.DOTALL)
+    for i in re.split(r'\n(?!\t)', stdout.decode('utf-8'), re.MULTILINE):
+        b = re.match(r'(\w+):.*status: (active|associated)$', i, re.MULTILINE | re.DOTALL)
         if b:
             yield b.group(1)
 
@@ -1868,8 +1866,8 @@ def unix_get_active_interfaces():
     """
     process = Popen(['ifconfig'], close_fds=True, stdout=subprocess.PIPE)
     stdout, _ = process.communicate()
-    for i in re.split('\n(?!\t)', stdout.decode(locale.getpreferredencoding()), re.MULTILINE):
-        b = re.match('(\\w+):.*status: (active|associated)$', i, re.MULTILINE | re.DOTALL)
+    for i in re.split(r'\n(?!\t)', stdout.decode(locale.getpreferredencoding()), re.MULTILINE):
+        b = re.match(r'(\w+):.*status: (active|associated)$', i, re.MULTILINE | re.DOTALL)
         if b:
             yield b.group(1)
 
@@ -1888,25 +1886,18 @@ def connection_available():
         elif gpodder.ui.osx:
             return len(list(osx_get_active_interfaces())) > 0
         else:
-            # By default, we assume we're not offline (bug 1730)
-            offline = False
+            # By default, we assume we're online (bug 1730)
+            online = True
 
-            if find_command('ifconfig') is not None:
+            if find_command('ip') is not None:
+                online = bool(list(linux_get_active_interfaces()))
+            elif find_command('ifconfig') is not None:
                 # If ifconfig is available, and it says we don't have
                 # any active interfaces, assume we're offline
-                if len(list(unix_get_active_interfaces())) == 0:
-                    offline = True
+                online = bool(list(unix_get_active_interfaces()))
 
-            # If we assume we're offline, try the "ip" command as fallback
-            if offline and find_command('ip') is not None:
-                if len(list(linux_get_active_interfaces())) == 0:
-                    offline = True
-                else:
-                    offline = False
+            return online
 
-            return not offline
-
-        return False
     except Exception as e:
         logger.warn('Cannot get connection status: %s', e, exc_info=True)
         # When we can't determine the connection status, act as if we're online (bug 1730)
@@ -1922,9 +1913,9 @@ def website_reachable(url):
         return (False, None)
 
     try:
-        response = urllib.request.urlopen(url, timeout=1)
+        response = requests.get(url, timeout=1)
         return (True, response)
-    except urllib.error.URLError as err:
+    except requests.exceptions.RequestException:
         pass
 
     return (False, None)
@@ -2182,3 +2173,42 @@ def parse_mimetype(mimetype):
     except MIMETypeException as e:
         print(e)
         return (None, None, {})
+
+
+def get_header_param(headers, param, header_name):
+    """Extract a HTTP header parameter from a dict
+
+    Uses the "email" module to retrieve parameters
+    from HTTP headers. This can be used to get the
+    "filename" parameter of the "content-disposition"
+    header for downloads to pick a good filename.
+
+    Returns None if the filename cannot be retrieved.
+    """
+    value = None
+    try:
+        headers_string = ['%s:%s' % (k, v) for k, v in list(headers.items())]
+        msg = email.message_from_string('\n'.join(headers_string))
+        if header_name in msg:
+            raw_value = msg.get_param(param, header=header_name)
+            if raw_value is not None:
+                value = email.utils.collapse_rfc2231_value(raw_value)
+    except Exception as e:
+        logger.error('Cannot get %s from %s', param, header_name, exc_info=True)
+
+    return value
+
+
+def response_text(response, default_encoding='utf-8'):
+    """
+    Utility method to return urlopen response's text.
+    Requests uses only the charset info in content-type, then defaults to ISO-8859-1
+    when content-type=text/*.
+    We could use chardet (via response.apparent_encoding) but it's slow so often it's
+    simpler to just use the known encoding.
+    :return: textual body of the response
+    """
+    if 'charset=' in response.headers.get('content-type'):
+        return response.text
+    else:
+        return response.content.decode(default_encoding)
